@@ -1,5 +1,6 @@
 import uuid
 import logging
+from laboro.context.step import Context as StepContext
 from laboro.workflow.step import Step
 
 
@@ -29,19 +30,22 @@ class Workflow:
 
       cfg_mgr = CfgMgr(main_config="/etc/laboro/laboro.yml")
       context = Context(logger=LogMgr(Vault()), config_mgr=cfg_mgr)
-      logging.getLogger("laboro.main").log_section("LABORO", "Bootstrapping")
-      logging.getLogger("laboro.main").vault.clear()
+      self.ctx.log.log_section("LABORO", "Bootstrapping")
+      self.ctx.log.vault.clear()
       cfg_mgr.workflow_config = "my_workflow.yml"
-      workflow = cfg_mgr.workflow_config
-      with Workflow(context=context, **workflow) as wkf:
+      workflow_config = cfg_mgr.workflow_config
+      with Workflow(context=context, **workflow_config) as wkf:
         wkf.run()
         ...
   """
-  def __init__(self, context, name, packages, workspace, steps):
+  def __init__(self, context, name, packages, workspace, steps, instances=None):
     self.name = name.replace(" ", "_")
     self.packages = packages
     self.workspace = workspace
     self.steps = steps
+    self.instances = instances
+    if instances is None:
+      self.instances = list()
     self.ctx = context.reset(session=str(uuid.uuid4()),
                              workspace_cfg=self.workspace)
 
@@ -51,9 +55,20 @@ class Workflow:
   def __exit__(self, kind, value, traceback):
     self.ctx.exit(kind, value)
 
+  def _instantiate(self, instance):
+    self.ctx.register_class(module=instance.get("module"),
+                            cls=instance.get("class"))
+    self.ctx.instantiate(module=instance.get("module"),
+                         cls=instance.get("class"),
+                         name=instance.get("name"),
+                         args=instance.get("args"))
+
   def run(self):
     """Run the workflow."""
+    for instance in self.instances:
+      self._instantiate(instance)
     for wkf_step in self.steps:
-      with Step(self.ctx, **wkf_step) as step:
+      with Step(StepContext(parent=self.ctx),
+                **wkf_step) as step:
         step.run()
-    logging.getLogger("laboro.main").log_line()
+    self.ctx.log.log_line()
