@@ -1,4 +1,5 @@
 import logging
+from laboro.context.action import Context as ActionContext
 from laboro.logic.processor import Processor
 from laboro.workflow.action import Action
 
@@ -35,9 +36,13 @@ class Step:
       return Processor().process(self.ctx, str(self.when))
     return True
 
-  def __init__(self, context, name, actions, when=None, loop=None):
+  def __init__(self, context, name, actions,
+               instances=None, when=None, loop=None):
     self.ctx = context
     self.name = name
+    self.instances = instances
+    if instances is None:
+      self.instances = list()
     self.when = when
     self.loop = loop
     self.actions = actions
@@ -48,18 +53,29 @@ class Step:
   def __exit__(self, kind, value, traceback):
     pass
 
+  def _instantiate(self, instance):
+    self.ctx.register_class(module=instance.get("module"),
+                            cls=instance.get("class"))
+    self.ctx.instantiate(module=instance.get("module"),
+                         cls=instance.get("class"),
+                         name=instance.get("name"),
+                         args=instance.get("args"))
+
   def run(self):
     """Run the step.
     Calling ``laboro.workflow.step.Step.run()`` will run all actions within the step if the ``runnable`` property evaluate to ``True``.
     The actions within the step will be executed for each item in the ``iterable`` property.
     """
+    for instance in self.instances:
+      self._instantiate(instance)
     for item in self.iterable:
-      self.ctx.store.step_item = item
+      self.ctx.set_step_item(item)
       if self.runnable:
-        logging.getLogger("laboro.main").log_section("STEP", self.name)
+        self.ctx.log.log_section("STEP", self.name)
         for step_action in self.actions:
-          with Action(self.ctx, **step_action) as action:
+          with Action(ActionContext(parent=self.ctx),
+                      **step_action) as action:
             action.run()
       else:
         msg = f"Skipping {self.name}: Condition not met: {self.when}"
-        logging.getLogger("laboro.main").log_section("STEP", msg, level=logging.WARNING)
+        self.ctx.log.log_section("STEP", msg, level=logging.WARNING)
